@@ -1,10 +1,6 @@
 use rand::Rng;
-use rnp::{
-    PingClientConfig, PingResultProcessorConfig, PingWorkerConfig, PingWorkerSchedulerConfig,
-    RnpCoreConfig,
-};
-use socket2::Protocol;
-use std::net::{IpAddr, SocketAddr, Ipv6Addr, Ipv4Addr};
+use rnp::{PingClientConfig, PingResultProcessorConfig, PingWorkerConfig, PingWorkerSchedulerConfig, RnpCoreConfig, RnpSupportedProtocol};
+use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr};
 use std::path::PathBuf;
 use std::time::Duration;
 use structopt::StructOpt;
@@ -41,10 +37,13 @@ pub struct RnpCliOptions {
     #[structopt(short = "t", help = "Ping until stopped.")]
     pub ping_until_stopped: bool,
 
+    #[structopt(long = "warmup", default_value = "1", help = "Warm up ping count.")]
+    pub warmup_count: u32,
+
     #[structopt(
         short = "w",
         long = "timeout",
-        default_value = "1000",
+        default_value = "2000",
         help = "Wait time for each ping in milliseconds."
     )]
     pub wait_timeout_in_ms: u32,
@@ -60,6 +59,9 @@ pub struct RnpCliOptions {
     #[structopt(long = "ttl", help = "Time to live.")]
     pub time_to_live: Option<u32>,
 
+    #[structopt(long = "use-fin", help = "Use regular TCP disconnect (w/ FIN) instead of RST.")]
+    pub use_fin_in_tcp_ping: bool,
+
     #[structopt(
         short = "p",
         long = "parallel",
@@ -123,13 +125,17 @@ impl RnpCliOptions {
     pub fn prepare_to_use(&mut self) {
         if self.target.is_ipv4() != self.source_ip.is_ipv4() {
             match &self.source_ip {
-                IpAddr::V4(source_ip_v4) if *source_ip_v4 == Ipv4Addr::UNSPECIFIED => self.source_ip = IpAddr::V6(Ipv6Addr::UNSPECIFIED),
-                IpAddr::V6(source_ip_v6) if *source_ip_v6 == Ipv6Addr::UNSPECIFIED => self.source_ip = IpAddr::V4(Ipv4Addr::UNSPECIFIED),
+                IpAddr::V4(source_ip_v4) if *source_ip_v4 == Ipv4Addr::UNSPECIFIED => {
+                    self.source_ip = IpAddr::V6(Ipv6Addr::UNSPECIFIED)
+                }
+                IpAddr::V6(source_ip_v6) if *source_ip_v6 == Ipv6Addr::UNSPECIFIED => {
+                    self.source_ip = IpAddr::V4(Ipv4Addr::UNSPECIFIED)
+                }
                 _ => panic!("Source IP and Target IP are not both IPv4 or IPv6!"),
             }
         }
         if self.source_port_min.is_none() {
-            self.source_port_min = Some(rand::thread_rng().gen_range(1024..20000));
+            self.source_port_min = Some(rand::thread_rng().gen_range(10000..30000));
         }
 
         if self.source_port_max.is_none() {
@@ -146,6 +152,20 @@ impl RnpCliOptions {
             self.ping_count = 1;
         }
 
+        let available_source_port_count = match &self.source_port_list {
+            Some(port_list) => port_list.len() as u32,
+            None => self.source_port_max.unwrap() as u32 - self.source_port_min.unwrap() as u32 + 1,
+        };
+
+        if self.parallel_ping_count > available_source_port_count {
+            tracing::warn!(
+                "Parallel ping count ({}) is larger than available source port count ({}), to avoid port conflict reducing parallel ping count down to the same as available source port count.",
+                self.parallel_ping_count,
+                available_source_port_count);
+
+            self.parallel_ping_count = available_source_port_count;
+        }
+
         if self.parallel_ping_count < 1 {
             tracing::warn!("Parallel ping count cannot be 0. Setting to 1 as minimum.");
             self.parallel_ping_count = 1;
@@ -153,7 +173,9 @@ impl RnpCliOptions {
 
         if let Some(latency_buckets) = &mut self.latency_buckets {
             tracing::debug!("Latency bucket set to 0. Use default one.");
-            if latency_buckets.len() == 0 || (latency_buckets.len() == 1 && latency_buckets[0] == 0.0) {
+            if latency_buckets.len() == 0
+                || (latency_buckets.len() == 1 && latency_buckets[0] == 0.0)
+            {
                 *latency_buckets = vec![0.1, 0.5, 1.0, 5.0, 10.0, 30.0, 50.0, 100.0, 300.0, 500.0];
             }
         }
@@ -162,13 +184,14 @@ impl RnpCliOptions {
     pub fn to_rnp_core_config(&self) -> RnpCoreConfig {
         let mut config = RnpCoreConfig {
             worker_config: PingWorkerConfig {
-                protocol: Protocol::TCP,
+                protocol: RnpSupportedProtocol::TCP,
                 target: self.target,
                 source_ip: self.source_ip,
                 ping_interval: Duration::from_millis(self.ping_interval_in_ms.into()),
                 ping_client_config: PingClientConfig {
                     wait_timeout: Duration::from_millis(self.wait_timeout_in_ms.into()),
                     time_to_live: self.time_to_live,
+                    use_fin_in_tcp_ping: self.use_fin_in_tcp_ping,
                 },
             },
             worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -179,6 +202,7 @@ impl RnpCliOptions {
                     None => None,
                 },
                 ping_count: None,
+                warmup_count: self.warmup_count,
                 parallel_ping_count: self.parallel_ping_count,
             },
             result_processor_config: PingResultProcessorConfig {
@@ -206,11 +230,7 @@ impl RnpCliOptions {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use rnp::{
-        PingClientConfig, PingResultProcessorConfig, PingWorkerConfig, PingWorkerSchedulerConfig,
-        RnpCoreConfig,
-    };
-    use socket2::Protocol;
+    use rnp::{PingClientConfig, PingResultProcessorConfig, PingWorkerConfig, PingWorkerSchedulerConfig, RnpCoreConfig, RnpSupportedProtocol};
     use std::path::PathBuf;
     use std::time::Duration;
     use structopt::StructOpt;
@@ -226,9 +246,11 @@ mod tests {
                 source_port_list: None,
                 ping_count: 4,
                 ping_until_stopped: false,
-                wait_timeout_in_ms: 1000,
+                warmup_count: 1,
+                wait_timeout_in_ms: 2000,
                 ping_interval_in_ms: 1000,
                 time_to_live: None,
+                use_fin_in_tcp_ping: false,
                 parallel_ping_count: 1,
                 no_console_log: false,
                 csv_log_path: None,
@@ -253,9 +275,11 @@ mod tests {
                 source_port_list: None,
                 ping_count: 10,
                 ping_until_stopped: true,
-                wait_timeout_in_ms: 2000,
+                warmup_count: 1,
+                wait_timeout_in_ms: 1000,
                 ping_interval_in_ms: 1500,
                 time_to_live: None,
+                use_fin_in_tcp_ping: false,
                 parallel_ping_count: 10,
                 no_console_log: true,
                 csv_log_path: None,
@@ -274,7 +298,7 @@ mod tests {
                 "10",
                 "-t",
                 "-w",
-                "2000",
+                "1000",
                 "-i",
                 "1500",
                 "-p",
@@ -299,9 +323,11 @@ mod tests {
                 source_port_list: Some(vec![1024, 1025, 1026]),
                 ping_count: 10,
                 ping_until_stopped: false,
-                wait_timeout_in_ms: 2000,
+                warmup_count: 3,
+                wait_timeout_in_ms: 1000,
                 ping_interval_in_ms: 1500,
                 time_to_live: Some(128),
+                use_fin_in_tcp_ping: true,
                 parallel_ping_count: 10,
                 no_console_log: true,
                 csv_log_path: Some(PathBuf::from("log.csv")),
@@ -324,12 +350,15 @@ mod tests {
                 "1024,1025,1026",
                 "--count",
                 "10",
+                "--warmup",
+                "3",
                 "--timeout",
-                "2000",
+                "1000",
                 "--interval",
                 "1500",
                 "--ttl",
                 "128",
+                "--use-fin",
                 "--parallel",
                 "10",
                 "--no-console-log",
@@ -352,13 +381,14 @@ mod tests {
         assert_eq!(
             RnpCoreConfig {
                 worker_config: PingWorkerConfig {
-                    protocol: Protocol::TCP,
+                    protocol: RnpSupportedProtocol::TCP,
                     target: "10.0.0.1:443".parse().unwrap(),
                     source_ip: "10.0.0.2".parse().unwrap(),
                     ping_interval: Duration::from_millis(1500),
                     ping_client_config: PingClientConfig {
-                        wait_timeout: Duration::from_millis(2000),
+                        wait_timeout: Duration::from_millis(1000),
                         time_to_live: Some(128),
+                        use_fin_in_tcp_ping: false,
                     },
                 },
                 worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -366,6 +396,7 @@ mod tests {
                     source_port_max: 2047,
                     source_port_list: Some(vec![1024, 1025, 1026]),
                     ping_count: Some(4),
+                    warmup_count: 1,
                     parallel_ping_count: 1,
                 },
                 result_processor_config: PingResultProcessorConfig {
@@ -382,13 +413,15 @@ mod tests {
                 target: "10.0.0.1:443".parse().unwrap(),
                 ping_count: 4,
                 ping_until_stopped: false,
+                warmup_count: 1,
                 source_ip: "10.0.0.2".parse().unwrap(),
                 source_port_min: Some(1024),
                 source_port_max: Some(2047),
                 source_port_list: Some(vec![1024, 1025, 1026]),
-                wait_timeout_in_ms: 2000,
+                wait_timeout_in_ms: 1000,
                 ping_interval_in_ms: 1500,
                 time_to_live: Some(128),
+                use_fin_in_tcp_ping: false,
                 parallel_ping_count: 1,
                 no_console_log: false,
                 csv_log_path: None,
@@ -404,13 +437,14 @@ mod tests {
         assert_eq!(
             RnpCoreConfig {
                 worker_config: PingWorkerConfig {
-                    protocol: Protocol::TCP,
+                    protocol: RnpSupportedProtocol::TCP,
                     target: "10.0.0.1:443".parse().unwrap(),
                     source_ip: "10.0.0.2".parse().unwrap(),
                     ping_interval: Duration::from_millis(1500),
                     ping_client_config: PingClientConfig {
                         wait_timeout: Duration::from_millis(2000),
                         time_to_live: Some(128),
+                        use_fin_in_tcp_ping: true,
                     },
                 },
                 worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -418,6 +452,7 @@ mod tests {
                     source_port_max: 2047,
                     source_port_list: Some(vec![1024, 1025, 1026]),
                     ping_count: None,
+                    warmup_count: 3,
                     parallel_ping_count: 1,
                 },
                 result_processor_config: PingResultProcessorConfig {
@@ -434,6 +469,7 @@ mod tests {
                 target: "10.0.0.1:443".parse().unwrap(),
                 ping_count: 4,
                 ping_until_stopped: true,
+                warmup_count: 3,
                 source_ip: "10.0.0.2".parse().unwrap(),
                 source_port_min: Some(1024),
                 source_port_max: Some(2047),
@@ -441,6 +477,7 @@ mod tests {
                 wait_timeout_in_ms: 2000,
                 ping_interval_in_ms: 1500,
                 time_to_live: Some(128),
+                use_fin_in_tcp_ping: true,
                 parallel_ping_count: 1,
                 no_console_log: true,
                 csv_log_path: Some(PathBuf::from("log.csv")),
@@ -482,9 +519,24 @@ mod tests {
         assert_eq!(1, opts.ping_count);
         assert_eq!(1, opts.parallel_ping_count);
         assert_eq!(
-            Some(vec![0.1, 0.5, 1.0, 5.0, 10.0, 30.0, 50.0, 100.0, 300.0, 500.0]),
+            Some(vec![
+                0.1, 0.5, 1.0, 5.0, 10.0, 30.0, 50.0, 100.0, 300.0, 500.0
+            ]),
             opts.latency_buckets
-        )
+        );
+
+        opts.source_port_min = Some(1024);
+        opts.source_port_max = Some(1047);
+        opts.parallel_ping_count = 100;
+        opts.prepare_to_use();
+        assert_eq!(24, opts.parallel_ping_count);
+
+        opts.source_port_min = None;
+        opts.source_port_max = None;
+        opts.source_port_list = Some(vec![1024, 1025, 1026]);
+        opts.parallel_ping_count = 100;
+        opts.prepare_to_use();
+        assert_eq!(3, opts.parallel_ping_count);
     }
 
     #[test]
