@@ -16,12 +16,4 @@ mod ping_result;
 mod ping_worker;
 mod rnp_core;
 mod rnp_core_config;
-mod rnp_utils;
-
-#[cfg(test)]
-mod tests {
-    #[test]
-    fn it_works() {
-        assert_eq!(2 + 2, 4);
-    }
-}
+mod rnp_utils;
\ No newline at end of file
