@@ -1,3 +1,6 @@
 pub mod ping_client_factory;
 pub mod ping_client;
-mod ping_client_tcp;
\ No newline at end of file
+mod ping_client_tcp;
+
+#[cfg(test)]
+mod ping_client_test_common;
\ No newline at end of file
