@@ -23,8 +23,9 @@ impl PingPortPicker {
         min_port: u16,
         max_port: u16,
         port_list: &Option<Vec<u16>>,
+        skip_port_count: u32,
     ) -> PingPortPicker {
-        return PingPortPicker {
+        let mut port_picker = PingPortPicker {
             remaining_ping_count: ping_count,
             min_port,
             max_port,
@@ -32,6 +33,12 @@ impl PingPortPicker {
             port_list: port_list.clone(),
             next_port_index: 0,
         };
+
+        for _ in 0..skip_port_count {
+            port_picker.next();
+        }
+
+        return port_picker;
     }
 
     fn fetch_next_available_port(&mut self) -> Option<u16> {
@@ -84,59 +91,62 @@ impl Iterator for PingPortPicker {
 
 #[cfg(test)]
 mod tests {
-    use crate::ping_port_picker::PingPortPicker;
+    use super::*;
 
     #[test]
     fn ping_port_picker_should_work_with_port_range_1() {
-        let picker = PingPortPicker::new(Some(3), 1024, 1024, &None);
-        assert_eq!(vec![1024, 1024, 1024], picker.collect::<Vec<u16>>());
+        assert_eq!(
+            vec![1024, 1024, 1024],
+            PingPortPicker::new(Some(3), 1024, 1024, &None, 0).collect::<Vec<u16>>()
+        );
     }
 
     #[test]
     fn ping_port_picker_should_work_with_limited_ping_count() {
-        let picker = PingPortPicker::new(Some(2), 1024, 1027, &None);
-        assert_eq!(vec![1024, 1025], picker.collect::<Vec<u16>>());
+        assert_eq!(
+            vec![1024, 1025],
+            PingPortPicker::new(Some(2), 1024, 1027, &None, 0).collect::<Vec<u16>>()
+        );
     }
 
     #[test]
     fn ping_port_picker_should_work_with_ping_count_larger_than_range() {
-        let picker = PingPortPicker::new(Some(6), 1024, 1027, &None);
         assert_eq!(
             vec![1024, 1025, 1026, 1027, 1024, 1025],
-            picker.collect::<Vec<u16>>()
+            PingPortPicker::new(Some(6), 1024, 1027, &None, 0).collect::<Vec<u16>>()
         );
     }
 
     #[test]
     #[should_panic]
     fn ping_port_picker_should_panic_on_zero_min_port() {
-        PingPortPicker::new(Some(3), 0, 1024, &None);
+        PingPortPicker::new(Some(3), 0, 1024, &None, 0);
     }
 
     #[test]
     #[should_panic]
     fn ping_port_picker_should_panic_on_zero_max_port() {
-        PingPortPicker::new(Some(3), 1024, 0, &None);
+        PingPortPicker::new(Some(3), 1024, 0, &None, 0);
     }
 
     #[test]
     #[should_panic]
     fn ping_port_picker_should_panic_when_min_port_is_larger_than_max_port() {
-        PingPortPicker::new(Some(3), 1028, 1024, &None);
+        PingPortPicker::new(Some(3), 1028, 1024, &None, 0);
     }
 
     #[test]
     fn ping_port_picker_should_work_with_port_list() {
-        let picker = PingPortPicker::new(Some(6), 1024, 1027, &Some(vec![1024, 1025, 1026]));
         assert_eq!(
-            vec![1024, 1025, 1026, 1024, 1025, 1026],
-            picker.collect::<Vec<u16>>()
+            vec![1024, 1025, 1026, 1024, 1025],
+            PingPortPicker::new(Some(5), 1024, 1027, &Some(vec![1024, 1025, 1026]), 0)
+                .collect::<Vec<u16>>()
         );
     }
 
     #[test]
     #[should_panic]
     fn ping_port_picker_should_panic_when_port_list_is_empty() {
-        PingPortPicker::new(Some(3), 1028, 1024, &Some(vec![]));
+        PingPortPicker::new(Some(3), 1028, 1024, &Some(vec![]), 0);
     }
-}
+}
\ No newline at end of file
