@@ -1,6 +1,5 @@
 use crate::ping_result_processors::ping_result_processor::PingResultProcessor;
 use crate::PingResult;
-use std::io;
 use std::time::Duration;
 use tracing;
 use contracts::requires;
@@ -44,13 +43,24 @@ impl PingResultProcessorLatencyBucketLogger {
     }
 
     fn update_statistics(&mut self, ping_result: &PingResult) {
+        // Skip warmup pings in analysis.
+        if ping_result.is_warmup() {
+            return;
+        }
+
+        // Skip preparation errors in analysis, since it is not a remote issue.
+        if ping_result.is_preparation_error() {
+            return;
+        }
+
         self.total_hit_count += 1;
 
-        // check time out / failures
-        match ping_result.error() {
-            Some(e) if e.kind() == io::ErrorKind::TimedOut => self.timed_out_hit_count += 1,
-            Some(_) => self.failed_hit_count += 1,
-            None => self.track_latency_in_buckets(&ping_result.round_trip_time()),
+        if ping_result.is_timed_out() {
+            self.timed_out_hit_count += 1;
+        } else if let Some(_) = ping_result.error() {
+            self.failed_hit_count += 1;
+        } else {
+            self.track_latency_in_buckets(&ping_result.round_trip_time());
         }
     }
 
@@ -68,11 +78,11 @@ impl PingResultProcessorLatencyBucketLogger {
 }
 
 impl PingResultProcessor for PingResultProcessorLatencyBucketLogger {
-    fn process(&mut self, ping_result: &PingResult) {
+    fn process_ping_result(&mut self, ping_result: &PingResult) {
         self.update_statistics(ping_result);
     }
 
-    fn done(&mut self) {
+    fn rundown(&mut self) {
         println!("\n=== Latency buckets (in milliseconds) ===\n");
         println!("{:>15} | {}", "Latency Range", "Count");
         println!("{:->17}------------ ", "+");
@@ -101,8 +111,8 @@ impl PingResultProcessor for PingResultProcessorLatencyBucketLogger {
 mod tests {
     use super::*;
     use chrono::{TimeZone, Utc};
-    use socket2::Protocol;
     use std::{io, time::Duration};
+    use crate::ping_clients::ping_client::PingClientError::{PreparationFailed, PingFailed};
 
     #[test]
     fn latency_bucket_logger_should_work() {
@@ -110,32 +120,52 @@ mod tests {
             PingResult::new(
                 &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
                 1,
-                Protocol::TCP,
+                "TCP",
                 "1.2.3.4:443".parse().unwrap(),
                 "5.6.7.8:8080".parse().unwrap(),
+                false,
                 Duration::from_millis(10),
+                false,
                 None,
             ),
             PingResult::new(
                 &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
                 1,
-                Protocol::TCP,
+                "TCP",
                 "1.2.3.4:443".parse().unwrap(),
                 "5.6.7.8:8080".parse().unwrap(),
+                false,
                 Duration::from_millis(1000),
-                Some(io::Error::new(io::ErrorKind::TimedOut, "timed out")),
+                true,
+                None,
             ),
             PingResult::new(
                 &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
                 1,
-                Protocol::TCP,
+                "TCP",
                 "1.2.3.4:443".parse().unwrap(),
                 "5.6.7.8:8080".parse().unwrap(),
+                false,
                 Duration::from_millis(0),
-                Some(io::Error::new(
+                false,
+                Some(PingFailed(Box::new(io::Error::new(
                     io::ErrorKind::ConnectionRefused,
                     "connect failed",
-                )),
+                )))),
+            ),
+            PingResult::new(
+                &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
+                1,
+                "TCP",
+                "1.2.3.4:443".parse().unwrap(),
+                "5.6.7.8:8080".parse().unwrap(),
+                false,
+                Duration::from_millis(0),
+                false,
+                Some(PreparationFailed(Box::new(io::Error::new(
+                    io::ErrorKind::AddrInUse,
+                    "address in use",
+                )))),
             ),
         ];
 
