@@ -1,10 +1,10 @@
+use crate::ping_clients::ping_client::{PingClientPingResultDetails, PingClientError};
 use crate::{ping_client_factory, PingClient, PingPortPicker, PingResult, PingWorkerConfig};
 use chrono::{offset::Utc, DateTime};
 use futures_intrusive::sync::ManualResetEvent;
-use socket2::SockAddr;
-use std::{io, net::SocketAddr, sync::Arc, sync::Mutex};
+use std::{net::SocketAddr, sync::Arc, sync::Mutex};
 use tokio::{sync::mpsc, task, task::JoinHandle};
-use crate::ping_clients::ping_client::PingClientPingResultDetails;
+use std::time::Duration;
 
 pub struct PingWorker {
     id: u32,
@@ -13,6 +13,7 @@ pub struct PingWorker {
     port_picker: Arc<Mutex<PingPortPicker>>,
     ping_client: Box<dyn PingClient + Send + Sync>,
     result_sender: mpsc::Sender<PingResult>,
+    is_warmup_worker: bool,
 }
 
 impl PingWorker {
@@ -27,18 +28,19 @@ impl PingWorker {
         port_picker: Arc<Mutex<PingPortPicker>>,
         stop_event: Arc<ManualResetEvent>,
         result_sender: mpsc::Sender<PingResult>,
+        is_warmup_worker: bool,
     ) -> JoinHandle<()> {
         let join_handle = task::spawn(async move {
-            let mut ping_client = ping_client_factory::new(config.protocol, &config.ping_client_config);
-            ping_client.prepare();
+            let ping_client = ping_client_factory::new(config.protocol, &config.ping_client_config);
 
-            let worker = PingWorker {
+            let mut worker = PingWorker {
                 id: worker_id,
                 config,
                 stop_event,
                 port_picker,
                 ping_client,
                 result_sender,
+                is_warmup_worker,
             };
             worker.run_worker_loop().await;
 
@@ -49,9 +51,13 @@ impl PingWorker {
     }
 
     #[tracing::instrument(name = "Running worker loop", level = "debug", skip(self), fields(worker_id = %self.id))]
-    async fn run_worker_loop(&self) {
+    async fn run_worker_loop(&mut self) {
         loop {
-            let source_port = self.port_picker.lock().expect("Failed getting port picker lock").next();
+            let source_port = self
+                .port_picker
+                .lock()
+                .expect("Failed getting port picker lock")
+                .next();
             match source_port {
                 Some(source_port) => self.run_single_ping(source_port).await,
                 None => {
@@ -67,38 +73,70 @@ impl PingWorker {
     }
 
     #[tracing::instrument(name = "Running single ping", level = "debug", skip(self), fields(worker_id = %self.id))]
-    async fn run_single_ping(&self, source_port: u16) {
-        let source = SockAddr::from(SocketAddr::new(self.config.source_ip, source_port));
-        let target = SockAddr::from(self.config.target);
-        let ping_time= Utc::now();
+    async fn run_single_ping(&mut self, source_port: u16) {
+        let source = SocketAddr::new(self.config.source_ip, source_port);
+        let target = self.config.target;
+        let ping_time = Utc::now();
+
         match self.ping_client.ping(&source, &target) {
-            Ok(result) => self.process_ping_client_result(&ping_time, source_port, result).await,
-            Err(result) => self.process_ping_client_result(&ping_time, source_port, result).await,
+            Ok(result) => {
+                self.process_ping_client_result(&ping_time, source_port, result)
+                    .await
+            }
+            Err(error) => {
+                self.process_ping_client_error(&ping_time, source_port, error)
+                    .await
+            }
         }
     }
 
     #[tracing::instrument(name = "Processing ping client single ping result", level = "debug", skip(self), fields(worker_id = %self.id))]
-    async fn process_ping_client_result(&self, ping_time: &DateTime<Utc>, src_port: u16, ping_result: PingClientPingResultDetails) {
-        // Failed due to unable to bind source port, the source port might be taken, and we should ignore this error and continue.
-        if let Some(e) = &ping_result.inner_error {
-            if e.kind() == io::ErrorKind::AddrInUse {
-                return;
-            }
-        }
+    async fn process_ping_client_result(
+        &self,
+        ping_time: &DateTime<Utc>,
+        src_port: u16,
+        ping_result: PingClientPingResultDetails,
+    ) {
+        let mut source: Option<SocketAddr> = ping_result.actual_local_addr;
 
-        let mut local_addr: Option<SocketAddr> = ping_result.actual_local_addr.and_then(|x| x.as_socket());
-        if local_addr.is_none() {
-            local_addr = Some(SocketAddr::new(self.config.source_ip, src_port));
+        if source.is_none() {
+            source = Some(SocketAddr::new(self.config.source_ip, src_port));
         }
 
         let result = PingResult::new(
             ping_time,
             self.id,
-            self.config.protocol,
+            self.ping_client.protocol(),
             self.config.target,
-            local_addr.unwrap(),
+            source.unwrap(),
+            self.is_warmup_worker,
             ping_result.round_trip_time,
-            ping_result.inner_error,
+            ping_result.is_timeout,
+            None,
+        );
+
+        self.result_sender.send(result).await.unwrap();
+    }
+
+    #[tracing::instrument(name = "Processing ping client single ping error", level = "debug", skip(self), fields(worker_id = %self.id))]
+    async fn process_ping_client_error(
+        &self,
+        ping_time: &DateTime<Utc>,
+        src_port: u16,
+        error: PingClientError,
+    ) {
+        let source = SocketAddr::new(self.config.source_ip, src_port);
+
+        let result = PingResult::new(
+            ping_time,
+            self.id,
+            self.ping_client.protocol(),
+            self.config.target,
+            source,
+            self.is_warmup_worker,
+            Duration::from_millis(0),
+            false,
+            Some(error),
         );
 
         self.result_sender.send(result).await.unwrap();
@@ -111,7 +149,10 @@ impl PingWorker {
 
         // Wait succedded, which means we are signaled to exit.
         if let Ok(_) = result {
-            tracing::debug!("Stop event received, stopping worker; worker_id={}", self.id);
+            tracing::debug!(
+                "Stop event received, stopping worker; worker_id={}",
+                self.id
+            );
             return false;
         }
 
