@@ -1,10 +1,14 @@
-use socket2::Protocol;
 use std::net::{IpAddr, SocketAddr};
 use std::{time::Duration, path::PathBuf};
 
 pub const RNP_NAME: &str = "rnp";
 pub const RNP_AUTHOR: &str = "r12f (r12f.com, github.com/r12f)";
-pub const RNP_ABOUT: &str = "A simple cloud-friendly tool for testing network reachability.";
+pub const RNP_ABOUT: &str = "A simple layer 4 ping tool for cloud.";
+
+#[derive(Debug, Copy, Clone, Ord, PartialOrd, Eq, PartialEq)]
+pub enum RnpSupportedProtocol {
+    TCP,
+}
 
 #[derive(Debug, Clone, PartialEq)]
 pub struct RnpCoreConfig {
@@ -15,7 +19,7 @@ pub struct RnpCoreConfig {
 
 #[derive(Debug, Clone, PartialEq)]
 pub struct PingWorkerConfig {
-    pub protocol: Protocol,
+    pub protocol: RnpSupportedProtocol,
     pub target: SocketAddr,
     pub source_ip: IpAddr,
     pub ping_interval: Duration,
@@ -26,6 +30,7 @@ pub struct PingWorkerConfig {
 pub struct PingClientConfig {
     pub wait_timeout: Duration,
     pub time_to_live: Option<u32>,
+    pub use_fin_in_tcp_ping: bool,
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -34,6 +39,7 @@ pub struct PingWorkerSchedulerConfig {
     pub source_port_max: u16,
     pub source_port_list: Option<Vec<u16>>,
     pub ping_count: Option<u32>,
+    pub warmup_count: u32,
     pub parallel_ping_count: u32,
 }
 
